<?php
/*
Plugin Name: Weather
Plugin URI: http://wpdemo.dovi42.hu/idojaras/
Description: Jelenlegi és várhatő időjárás megjelenítése (http://openweathermap.org/)
Author: Dovi
Version: 2.15
Author URI: http://dovi.hu/
*/

// Don't call the file directly
if ( !defined( 'ABSPATH' ) ) exit;

require_once ( dirname(__file__) . DIRECTORY_SEPARATOR . 'autoupdate.php' ); 


function DoviWeatherWidget_activate_au()
{
	// set auto-update params
	$plugin_current_version = '2.15'; // Current version
	$plugin_remote_path     = 'http://update.dovi42.hu/dovi-weather/update.php';  // <remote path to your update server> e.g. http://update.example.com';
	$plugin_slug            = plugin_basename(__FILE__);
	$license_user           = get_site_url();
	$license_key            = '<optional license key>';

	// only perform Auto-Update call if a license_user and license_key is given
	if ( $license_user && $license_key && $plugin_remote_path )
	{
		new DoviWeatherWidget_autoupdate ($plugin_current_version, $plugin_remote_path, $plugin_slug, $license_user, $license_key);
	}
}
add_action('init', 'DoviWeatherWidget_activate_au');

function dovi_weather_register_widget() {
    register_widget( 'Dovi_Weather_Widget' );
}
add_action( 'widgets_init', 'dovi_weather_register_widget' );


/**
 * Register our styles
 *
 * @return void
 */
function dovi_weather_widget_register_widget_styles() {

	wp_register_style( 'dovi-weather-css', plugins_url( 'dovi-weather/assets/css/dovi-weather.css' ) );
	wp_enqueue_style( 'dovi-weather-css' );
    
}
add_action( 'wp_enqueue_scripts', 'dovi_weather_widget_register_widget_styles' );

function dovi_weather_widget_register_widget_js() {

	wp_register_script('dovi-weather-js', plugins_url( 'dovi-weather/assets/js/dovi-weather.min.js' ), array('jquery'), false, true);
	wp_enqueue_script('dovi-weather-js');
    wp_localize_script('dovi-weather-js', 'dovi_weather_ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
}
// add_action( 'wp_enqueue_scripts', 'dovi_weather_widget_register_widget_js' );


/**
 * Load plugin textdomain.
 *
 * @since 1.0.0
 */
function dovi_weather_widget_load_textdomain() {
  load_plugin_textdomain( 'dovi-weather', false, basename( dirname( __FILE__ ) ) . '/languages' ); 
}
add_action( 'plugins_loaded', 'dovi_weather_widget_load_textdomain' );

function dovi_weather_getAlias($string)
{
    $str = trim(mb_strtolower($string, 'UTF-8'));

    $accented = array( "á", "é", "í", "ó", "ö", "ő", "ú", "ü", "ű" );  
    $mask = array( "a", "e", "i", "o", "o", "o", "u", "u", "u" );  
    $str = str_replace( $accented, $mask, $str );  

    $str = str_replace('-', ' ', $str);
    $str = preg_replace(array('/\s+/','/[^A-Za-z0-9\-]/'), array('-',''), $str);
    
    return $str;
}          

function dovi_weather_getToken()
{
    $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
    $token = '';
    for ($i = 0; $i <= 4; $i++) {
    	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
    }                
    return $token;
}


function get_current_weather_html($data) {

    if (empty($data)) {
        return '<!-- Current weather data is empty -->';
    }

    if (function_exists('child_get_current_weather_html')) {
        $html = child_get_current_weather_html($data);    
    } else {
        $html = '<img class="dovi-weather-icon" src="http://openweathermap.org/img/wn/'.$data['icon'].'@2x.png" />'.
                '<span class="dovi-weather-city">'.$data['city'].'</span><br /><span class="dovi-weather-description">'.$data['desc'].', '.$data['temp'].__('Celsius', 'dovi-weather').'</span>' . 
                '<!-- '.$data['timestamp'].' -->'; 
    }
    return $html;
}

function get_forecast_weather_html($data) {

    if (empty($data)) {
        return '<!-- Forecast weather data is empty -->';
    }

    if (function_exists('child_get_forecast_weather_html')) {
        $html = child_get_forecast_weather_html($data);
    } else {
        $html = '<table><tr>';
        for ($i = 0; $i <= count($data)-1; $i++) {
            $item = $data[$i];
            $html .= '<td>' . 
                     '<img class="dovi-weather-forecast-icon" src="http://openweathermap.org/img/wn/'.$item['icon'].'@2x.png" title="'.$item['desc'].'" /><br />'.
                     '<div class="dovi-weather-today">'. $item['time'] . ' ' . __('hour', 'dovi-weather') . '</div>' .
                     '<div class="dovi-weather-description">' . 
                     $item['desc'] . '<br />' .
                     '</div>'. 
                     '<div class="dovi-weather-temp">' . 
                     $item['temp'] . __('Celsius', 'dovi-weather') . 
                     '</div>'. 
                     '</td>';
        }
        $html .= '</tr></table>';
    }
    return $html;
}



class Dovi_Weather_Widget extends WP_Widget {

	// Widgetek közötti megjelenést szabályozó rész
    function __construct() {
		$widget_ops = array( 'classname'   => 'dovi-weather', 'description' => __( 'Jelenlegi és várható időjárás', 'dovi-weather' ) );
		parent::__construct( 'dovi-weather', __( 'Időjárás', 'dovi-weather' ), $widget_ops );
	}

	// Displays latest posts widget on blog.
	function widget( $args, $options ) {
		global $post;

        if (($options['api_key'] == '') or ($options['city'] == '')) {
            return '';
        }

        $url = array();
        
        $city = $options['city'];
        if ($options['country'] != '') {
            $city .= ',' . $options['country'];
        }
        $url[] = 'q=' . $city;
        $url[] = 'units=metric';
        if ($options['lang'] != '') {
            $url[] = 'lang=' . $options['lang'];
        }
        $url[] = 'cnt=4';
        $url[] = 'appid=' . $options['api_key'];
        
        $alias = dovi_weather_getAlias($options['city']);

        $current  = 'https://api.openweathermap.org/data/2.5/weather?' . implode('&', $url);
        $forecast = 'https://api.openweathermap.org/data/2.5/forecast?' . implode('&', $url);

        $current_html = dovi_weather_checkValid('current', $alias);
        $forecast_html = dovi_weather_checkValid('forecast', $alias);
        $needUpdate = '';
        $data_current = '';
        $data_forecast = '';
        if (($current_html == '') or ($forecast_html == '')) {
            dovi_weather_widget_register_widget_js();
            $img = '      <img src="'.plugins_url( 'dovi-weather/assets/images/wait.gif' ).'" style="height: 24px;" />' . "\r\n";
            $needUpdate = 'dovi-weather-need-update';
            $data_current = 'data-current="'.$current.'"';
            $data_forecast = 'data-forecast="'.$forecast.'"';
            
        } else {
            $img = '';
        }

        extract( $args );

        $html = $before_widget;
        $html .= '<div id="dovi_weather_'.$alias.'_'.dovi_weather_getToken().'" class="dovi-weather '.$needUpdate.'" data-alias="'.$alias.'" '.$data_current.' '.$data_forecast.'>' . "\r\n";
        $title = $options['title'];
        if (($title != '') and ($options['showtitle'])) {
            $html .= $before_title . $title . $after_title;
        } 
        $html .= '  <div class="dovi-weather-data">' . "\r\n" . $img;
        if ($options['show_now'] == 1) {
            $html .= '      <div class="dovi-weather-current">'.
                     $current_html . 
                     '      </div>'.
                     '      <div style="clear:both;"></div>';
        }
        if ($options['show_forecast'] == 1) {
            $html .= '      <div class="dovi-weather-forecast">'.
                     $forecast_html . 
                     '      </div>'.
                     '      <div style="clear:both;"></div>';
        }
        $html .= '  </div>' . "\r\n" .
                 '</div>' . "\r\n" .
                 '<div style="clear:both;"></div>' . "\r\n";
        $html .= $after_widget;
        echo $html;

	}


	function form( $options ) {

		$options = wp_parse_args( ( array ) $options, array(
			'title'     => __( 'Időjárás', 'dovi-weather' ),
			'showtitle' => 1,
			'api_key'   => '',
			'lang'      => __( 'hu' , 'dovi-weather'),
			'city'      => '',
			'country'   => __( 'hu' , 'dovi-weather'),
			'show_now'  => 1,
			'show_forecast'  => 1,
            'timestamp' => date('Y-m-d H:i:s', strtotime('-2 hour'))
		) );

		?>

		<p>
			<label for="<?php echo $this->get_field_id( "title" ); ?>">
				<?php _e( 'Title', 'dovi-weather' ); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id( "title" ); ?>" name="<?php echo $this->get_field_name( "title" ); ?>" type="text" value="<?php echo esc_attr( $options["title"] ); ?>" />
			</label>
            <?php
            if ($options['showtitle'] == 1) {
                $checked = ' CHECKED ';
            } else {
                $checked = '';    
            } 
            ?>
                <input type="checkbox" id="<?php echo $this->get_field_id( "showtitle" );?>" name="<?php echo $this->get_field_name( "showtitle" );?>" value="1" <?php echo $checked;?> />
                <?php echo __('Show title.', 'dovi-weather');?>
		</p>
		<div>
			<label for="<?php echo $this->get_field_id( "api_key" ); ?>">
				<?php _e( 'API KEY', 'dovi-weather' ); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id( "api_key" ); ?>" name="<?php echo $this->get_field_name( "api_key" ); ?>" type="text" value="<?php echo esc_attr( $options["api_key"] ); ?>" />
                <div style="font-size: 90%;"><?php echo __('Get key from <a href="https://openweathermap.org/api" target="_blank">openweathermap.org</a>', 'dovi-weather');?></div>
			</label>
        </div>
		<div style="margin-top: 12px;">
			<label for="<?php echo $this->get_field_id( "lang" ); ?>">
				<?php _e( 'Lang code', 'dovi-weather' ); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id( "lang" ); ?>" name="<?php echo $this->get_field_name( "lang" ); ?>" type="text" value="<?php echo esc_attr( $options["lang"] ); ?>" />
                <div style="font-size: 90%;"><?php echo __('Set to English: en', 'dovi-weather');?></div>
			</label>
        </div>
		<p>
			<label for="<?php echo $this->get_field_id( "city" ); ?>">
				<?php _e( 'City name', 'dovi-weather' ); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id( "city" ); ?>" name="<?php echo $this->get_field_name( "city" ); ?>" type="text" value="<?php echo esc_attr( $options["city"] ); ?>" />
			</label>
        </p>
		<div>
			<label for="<?php echo $this->get_field_id( "country" ); ?>">
				<?php _e( 'Country code', 'dovi-weather' ); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id( "country" ); ?>" name="<?php echo $this->get_field_name( "country" ); ?>" type="text" value="<?php echo esc_attr( $options["country"] ); ?>" />
                <div style="font-size: 90%;"><?php echo __('Set to British: en', 'dovi-weather');?></div>
			</label>
        </div>
		<p>
            <?php
            if ($options['show_now'] == 1) {
                $checked = ' CHECKED ';
            } else {
                $checked = '';    
            } 
            ?>
                <input type="checkbox" id="<?php echo $this->get_field_id( "show_now" );?>" name="<?php echo $this->get_field_name( "show_now" );?>" value="1" <?php echo $checked;?> />
                <?php echo __('Show current values.', 'dovi-weather');?>
		</p>
		<p>
            <?php
            if ($options['show_forecast'] == 1) {
                $checked = ' CHECKED ';
            } else {
                $checked = '';    
            } 
            ?>
                <input type="checkbox" id="<?php echo $this->get_field_id( "show_forecast" );?>" name="<?php echo $this->get_field_name( "show_forecast" );?>" value="1" <?php echo $checked;?> />
                <?php echo __('Show forecast values.', 'dovi-weather');?>
		</p>
		<?php
	}

	function update( $new_options, $old_options ) {

        if (isset($new_options['title'])) {
            $new_options['title'] = trim($new_options['title']);
        }

        if (isset($new_options['showtitle'])) {
            $new_options['showtitle'] = 1;
        } else {
            $new_options['showtitle'] = 0;
        }

        if (isset($new_options['api_key'])) {
            $new_options['api_key'] = trim($new_options['api_key']);
        }

        if (isset($new_options['lang'])) {
            $new_options['lang'] = trim($new_options['lang']);
        }

        if (isset($new_options['city'])) {
            $new_options['city'] = trim($new_options['city']);
        }

        if (isset($new_options['country'])) {
            $new_options['country'] = trim($new_options['country']);
        }

        if (isset($new_options['show_now'])) {
            $new_options['show_now'] = 1;
        } else {
            $new_options['show_now'] = 0;
        }

        if (isset($new_options['show_forecast'])) {
            $new_options['show_forecast'] = 1;
        } else {
            $new_options['show_forecast'] = 0;
        }

		return $new_options;
	}
}

function dovi_weather_checkValid($source, $alias)
{
    $data = get_option('dovi_weather_' . $source . '_' . $alias);
    if (!isset($data['timestamp']) or ($data['timestamp'] <= date('Y-m-d H:i:s', strtotime('-1 hour')))) {
        return '';
    }
    if ($source == 'current') {
        return get_current_weather_html($data);
    } else {
        return get_forecast_weather_html($data['hours']);
    }
}

function get_dovi_weather_content() {

    $current  = $_REQUEST['current'];    
    $forecast = $_REQUEST['forecast'];
    $alias    = $_REQUEST['alias'];
    $id       = $_REQUEST['id'];

    
    $current_data = get_option('dovi_weather_current_'.$alias);
    if (!isset($current_data['timestamp']) or ($current_data['timestamp'] <= date('Y-m-d H:i:s', strtotime('-1 hour'))) or ($current_data == null)) {
        $data = file_get_contents($current);
        $data = json_decode($data, true);

        if (!empty($data)) {
        
            $current_data = array(
                    'icon' => $data['weather'][0]['icon'],
                    'city' => $data['name'],
                    'desc' => $data['weather'][0]['description'],
                    'temp' => round($data['main']['temp']),
                    'timestamp' => date('Y-m-d H:i:s')
                );
        } else {
            $current_data = null;
        }
        update_option('dovi_weather_current_'.$alias, $current_data, true);

    }
    $current_html = get_current_weather_html($current_data);


    $forecast_data = get_option('dovi_weather_forecast_'.$alias);
    if (!isset($forecast_data['timestamp']) or ($forecast_data['timestamp'] <= date('Y-m-d H:i:s', strtotime('-1 hour'))) or ($forecast_data == null)) {
        $data = file_get_contents($forecast);
        $data = json_decode($data, true);
        
        if (!empty($data)) {
            
            $forecast_data = array(
                    'timestamp' => date('Y-m-d H:i:s'),
                    'hours'     => array()
                );
            for ($i = 1; $i <= count($data['list'])-1; $i++) {
                $item = $data['list'][$i];
                
                $time = strftime('%H', $item['dt']);
                if ($time < 10) { $time = substr($time, 1); }
                
                $temp = array(
                        'time' => $time,
                        'icon' => $item['weather'][0]['icon'],
                        'desc' => $item['weather'][0]['description'],
                        'temp' => round($item['main']['temp'])                    
                    );   
                $forecast_data['hours'][] = $temp;
            }
        } else {
            $forecast_data = null;
        }
        update_option('dovi_weather_forecast_'.$alias, $forecast_data, true);
    }
    $forecast_html = get_forecast_weather_html($forecast_data['hours']);
    
    $data = array(
            'status' => 1,
            'current'  => $current_html,
            'forecast' => $forecast_html,
            'id'       => $id
        );
    echo json_encode($data);          
    wp_die();
}
add_action( 'wp_ajax_nopriv_get_dovi_weather_content', 'get_dovi_weather_content' );
add_action( 'wp_ajax_get_dovi_weather_content', 'get_dovi_weather_content' );
